@@ -183,8 +183,13 @@ class Repository(_Base):
         else:
             raise AttributeError
 
-    def clone(self):  # pragma: no cover
-        raise NotImplementedError
+    def clone(self, name, autosync=True):
+        """Clone the repository.
+        @param name: name of cloned repository
+        @param autosync: enable automatic pull/push between the clones
+        @return: new Repository
+        """
+        return Repository(name, remote=self, create=True, autosync=autosync)
 
     def incoming(self):  # pragma: no cover
         raise NotImplementedError
@@ -242,10 +247,8 @@ class Repository(_Base):
         """If enabled, perform an automatic pull and push.
         """
         if self._auto_pull:
-            self._pull_time = 0
             self.pull(_auto=True)
         if self._auto_push:
-            self._push_time = 0
             self.push(_auto=True)
 
     @property
@@ -581,7 +584,7 @@ class Repository(_Base):
         @param name: name for the build
         @param series: alias of a build series
         @param environment: alias of an environment
-        @param status: alias of the initial build state
+        @param status: alias of the initial build status
         @param cid: changeset identifier (unique revision)
         @param branch: optional branch name to associate with the build
         @return new BuildRequest
@@ -589,7 +592,8 @@ class Repository(_Base):
         logging.info("requesting [{s}] build of '{c}' for [{e}] in '{r}'...".format(s=series, e=environment,
                                                                                     c=cid, r=self))
         args = [name, series, environment, status, cid] + ([branch] if branch else [])
-        bid, cid = next(vscript.parse(vscript.report_build(self, 'new', *args), style='builds'))
+        bid, cid, _name, _series, _environment = next(vscript.parse(vscript.report_build(self, 'new', *args),
+                                                                    style='builds'))
         return BuildRequest(bid, cid, name=name, series=series, environment=environment,
                             status=status, branch=branch, repo=self)
 
@@ -597,24 +601,24 @@ class Repository(_Base):
     def get_build(self, environment, status, order=None):
         """Get the next pending build request in the repository.
         @param environment: alias for the current environment
-        @param status: alias for the initial build state
+        @param status: alias for the initial build status
         @param order: return the 'OLDEST' or 'NEWEST' request
         @return: pending BuildRequest
         """
         logging.debug("getting the next build request for [{e}] in '{r}'...".format(e=environment, r=self))
-        args = [environment, order or BuildRequest.OLDEST, status]
+        args = self, 'get-pending-build', environment, order or BuildRequest.OLDEST, status
         try:
-            bid, cid = next(vscript.parse(vscript.report_build(self, 'get-pending-build', *args), style='builds'))
+            bid, cid, name, series, environment = next(vscript.parse(vscript.report_build(*args), style='builds'))
         except StopIteration:
             return None
         else:
-            return BuildRequest(bid, cid, environment=environment, status=status, repo=self)
+            return BuildRequest(bid, cid, name=name, series=series, environment=environment, status=status, repo=self)
 
     @auto_push
     def update_build(self, bid, status):
-        """Update the build request state in the repository.
+        """Update the build request status in the repository.
         @param bid: build request identifier
-        @param status: alias for a build state
+        @param status: alias for a build status
         """
         logging.info("updating build '{b}' to status '{s}' in '{r}'...".format(b=bid, s=status, r=self))
         vscript.report_build(self, 'update', bid, status)
@@ -627,18 +631,21 @@ class Repository(_Base):
         logging.info("deleting build '{b}' in '{r}'...".format(b=bid, r=self))
         vscript.report_build(self, 'delete', bid)
 
-    @property
     @auto_pull
-    def builds(self):
+    def get_builds(self, limit=settings.MAX_BUILDS):
         """Get a list of active build requests in the repository.
         @return: iterator of BuildRequests
         """
         logging.debug("getting all build requests in '{r}'...".format(r=self))
-        for bid, cid in vscript.parse(vscript.report_build(self, 'list'), style='builds'):
-            if bid and cid:
-                request = BuildRequest(bid, cid, repo=self)
-                logging.debug("build request: {0}".format(request))
-                yield request
+        if limit:
+            args = self, 'list', 'null', 'null', limit
+        else:
+            args = self, 'list'
+        for bid, cid, name, series, environment in vscript.parse(vscript.report_build(*args), style='builds'):
+            request = BuildRequest(bid, cid, name=name, series=series, environment=environment, repo=self)
+            logging.debug("build request: {0}".format(request))
+            yield request
+    builds = property(get_builds, doc='')
 
     @builds.deleter
     @auto_pull
@@ -1249,12 +1256,12 @@ class BuildRequest(_BaseRepositoryObject):
         @param name: name for the build
         @param series: alias of a build series
         @param environment: alias of an environment
-        @param status: alias of the initial build state
+        @param status: alias of the initial build status
         @param branch: optional branch name to associate with the build
         @param kwargs: repo
         """
         super(BuildRequest, self).__init__(**kwargs)
-        self.bid = bid or '?' * 10
+        self.bid = bid
         self.changeset = Changeset(cid, repo=self.repo)
         # TODO: determine which attributes are actually needed by the class
         self.name = name
@@ -1285,8 +1292,8 @@ class BuildRequest(_BaseRepositoryObject):
         return not self == other
 
     def update(self, status):
-        """Update the build state.
-        @param status: alias for a build state
+        """Update the build status.
+        @param status: alias for a build status
         """
         self.repo.update_build(self.bid, status)
         self.status = status
