@@ -9,6 +9,7 @@ import os
 import shutil
 import tempfile
 import logging
+from random import randrange
 
 from veracity import vv
 from veracity.vv import VeracityException
@@ -18,7 +19,24 @@ from veracity.objects import Repository, WorkingCopy, Item, Changeset, Branch, T
 from veracity.objects import BuildRequest
 from veracity import settings
 
-from veracity.test import TEST_REPO_NAME, TEMP_REPO_NAME, TEST_REPO, REASON, log
+from veracity.test import TEST_REPO, REASON, log
+
+
+def setUpModule():  # pylint: disable=C0103
+    """Create a temporary, randomly-named repository for this module's tests.
+    """
+    global TEMP_REPO  # pylint: disable=W0601
+    if TEST_REPO:
+        TEMP_REPO = TEST_REPO.clone(TEST_REPO.name + '-' + str(randrange(999)), autosync=False)
+    else:
+        TEMP_REPO = None
+
+
+def tearDownModule():  # pylint: disable=C0103
+    """Delete the temporary repository.
+    """
+    if TEMP_REPO:
+        TEMP_REPO.delete()
 
 
 @unittest.skipUnless(TEST_REPO, REASON)  # pylint: disable=R0904
@@ -27,44 +45,41 @@ class TestRepository(unittest.TestCase):
 
     def test_init_existing(self):
         """Verify a repository can be created from an existing repository."""
-        repo = Repository(TEST_REPO_NAME)
+        repo = Repository(TEST_REPO.name)
         log(repo)
         self.assertEqual(TEST_REPO, repo)
-        self.assertEqual(TEST_REPO_NAME, repo)
+        self.assertEqual(TEST_REPO.name, repo)
         self.assertNotEqual("", repo)
         self.assertNotEqual(None, repo)
 
     def test_init_clone(self):
         """Verify a repository can be created as a clone."""
-        with Repository(TEMP_REPO_NAME, remote=TEST_REPO_NAME, autosync=False) as repo:
+        with Repository(TEMP_REPO.name + '-clone', remote=TEMP_REPO) as repo:
             log(repo)
-            self.assertEqual(TEMP_REPO_NAME, str(repo))
-            with Repository(TEMP_REPO_NAME + '-clone', remote=repo) as repo2:
-                log(repo2)
 
-                repo2._push_time = 0  # force push, pylint: disable=W0212
-                repo2.add_tag('_temptag2', branch=Branch.MASTER)  # tag will get pushed to TEST_REPO
-                self.assertIn('_temptag2', repo.tags)
+            repo._push_time = 0  # force push, pylint: disable=W0212
+            repo.add_tag('_temptag2', branch=Branch.MASTER)  # tag will get pushed to TEMP_REPO
+            self.assertIn('_temptag2', TEMP_REPO.tags)
 
-                repo.add_tag('_temptag', branch=Branch.MASTER)
-                repo2._pull_time = 0  # force push, pylint: disable=W0212
-                self.assertIn('_temptag', repo2.tags)  # tag will get pulled  from TEST_REPO
-                self.assertIn('_temptag', repo.tags)
+            TEMP_REPO.add_tag('_temptag', branch=Branch.MASTER)
+            repo._pull_time = 0  # force push, pylint: disable=W0212
+            self.assertIn('_temptag', repo.tags)  # tag will get pulled  from TEMP_REPO
+            self.assertIn('_temptag', TEMP_REPO.tags)
 
     def test_init_new(self):
         """Verify a new repository can be created."""
-        with Repository(TEMP_REPO_NAME, create=True) as repo:
+        with Repository(TEMP_REPO.name + '-new', create=True) as repo:
             log(repo)
-            self.assertEqual(TEMP_REPO_NAME, str(repo))
+            self.assertEqual(TEMP_REPO.name + '-new', str(repo))
 
     def test_init_new_no_create(self):
         """Verify an exception occurs when creation of new repositories is disabled."""
-        self.assertRaises(VeracityException, Repository, TEMP_REPO_NAME, create=False)
+        self.assertRaises(VeracityException, Repository, TEMP_REPO.name + '-new', create=False)
 
     def test_delete(self):
         """Verify a repository can not be used after it is deleted."""
-        repo2 = Repository(TEMP_REPO_NAME, remote=TEST_REPO_NAME)
-        with Repository(TEMP_REPO_NAME, remote=TEST_REPO_NAME) as repo:
+        repo2 = Repository(TEMP_REPO.name + '-clone', remote=TEMP_REPO.name)
+        with Repository(TEMP_REPO.name + '-clone', remote=TEMP_REPO.name) as repo:
             log(repo, repo2)
             self.assertEqual(repo, repo2)
         self.assertRaises(VeracityException, repo.delete)
@@ -92,7 +107,7 @@ class TestWorkingCopy(unittest.TestCase):
 
     def test_init_existing(self):
         """Verify a working copy can be checked out from an existing repository."""
-        with WorkingCopy(tempfile.mkdtemp(), repo=TEST_REPO_NAME) as wc:
+        with WorkingCopy(tempfile.mkdtemp(), repo=TEST_REPO.name) as wc:
             log(wc)
             self.assertTrue(os.path.isdir(wc.path))
             self.assertEqual(wc.path, str(wc))
@@ -100,7 +115,7 @@ class TestWorkingCopy(unittest.TestCase):
     def test_init_path(self):
         """Verify a working copy can be creating from an existing path."""
         temp = tempfile.mkdtemp()
-        with WorkingCopy(temp, repo=TEST_REPO_NAME) as wc1:
+        with WorkingCopy(temp, repo=TEST_REPO.name) as wc1:
             with WorkingCopy(wc1.path) as wc2:
                 log(wc1, wc2)
                 self.assertEqual(wc1, wc2)
@@ -110,7 +125,7 @@ class TestWorkingCopy(unittest.TestCase):
         """Verify a working copy can be checked out to a new path."""
         temp = tempfile.mkdtemp()
         shutil.rmtree(temp)  # force the checkout to create this directory
-        with WorkingCopy(temp, repo=TEST_REPO_NAME) as wc:
+        with WorkingCopy(temp, repo=TEST_REPO.name) as wc:
             log(wc)
             self.assertEqual(temp, wc.path)
 
@@ -191,7 +206,7 @@ class TestChangeset(unittest.TestCase):
     def test_eq(self):
         """Verify changests can be compared."""
         cs1 = Changeset('abc123', repo=TEST_REPO)
-        cs2 = Changeset('abc123', repo=TEST_REPO_NAME)
+        cs2 = Changeset('abc123', repo=TEST_REPO.name)
         cs3 = Changeset('abc123', repo=None)
         cs4 = Changeset('def456', repo=TEST_REPO)
         log(cs1, cs2, cs3, cs4)
@@ -206,25 +221,15 @@ class TestChangeset(unittest.TestCase):
 class TestBranch(unittest.TestCase):
     """Tests on the Branch class."""  # pylint: disable=C0103
 
-    @classmethod
-    def setUpClass(cls):
-        if TEST_REPO:
-            cls.repo = Repository(TEMP_REPO_NAME, remote=TEST_REPO, autosync=False)
-
-    @classmethod
-    def tearDownClass(cls):
-        if TEST_REPO:
-            cls.repo.delete()
-
     def test_repo_branches(self):
         """Verify the branches list can be retrieved from a repo."""
-        branches = self.repo.branches
+        branches = TEMP_REPO.branches
         log(branches)
         self.assertIn('master', branches)
 
     def test_repo_add_move_branch(self):
         """Verify a branch can be added and moved."""
-        b = self.repo.add_branch('_temp_branch', branch='master')
+        b = TEMP_REPO.add_branch('_temp_branch', branch='master')
         try:
             self.assertEqual('master', b.branch)
             b = b.move(rev=1)
@@ -237,11 +242,11 @@ class TestBranch(unittest.TestCase):
 
     def test_repo_close_open_branch(self):
         """Verify a branch can be closed and opened"""
-        b = Branch('master', repo=self.repo, branch='master')
+        b = Branch('master', repo=TEMP_REPO, branch='master')
         b.close()
         try:
             b.close()  # already closed
-            self.assertNotIn(b, self.repo.branches)
+            self.assertNotIn(b, TEMP_REPO.branches)
             b.reopen()
         except:
             logging.error("exception will be raised after cleanup...")
@@ -251,8 +256,8 @@ class TestBranch(unittest.TestCase):
 
     def test_eq(self):
         """Verify branches can be compared."""
-        b1 = Branch('a', repo=self.repo, rev=1)
-        b2 = Branch('b', repo=self.repo, rev=1)
+        b1 = Branch('a', repo=TEMP_REPO, rev=1)
+        b2 = Branch('b', repo=TEMP_REPO, rev=1)
         b3 = Branch('b', repo=None, rev=1)
         b4 = Branch('b', repo=None, rev=1)
         log(b1, b2, b3, b4)
@@ -263,7 +268,7 @@ class TestBranch(unittest.TestCase):
 
     def test_wc_branch(self):
         """Verify the branch can be set on a working copy."""
-        with self.repo.checkout(tempfile.mkdtemp()) as wc:
+        with TEMP_REPO.checkout(tempfile.mkdtemp()) as wc:
             wc.branch = '_new_branch'
             self.assertEqual('_new_branch', wc.branch)
             branch = wc.attach('master')
@@ -271,7 +276,7 @@ class TestBranch(unittest.TestCase):
 
     def test_wc_branch_none(self):
         """Verify setting the working copy's branch to None detaches it."""
-        with self.repo.checkout(tempfile.mkdtemp()) as wc:
+        with TEMP_REPO.checkout(tempfile.mkdtemp()) as wc:
             wc.branch = None
             self.assertEqual(None, wc.branch)
 
@@ -283,14 +288,12 @@ class TestItem(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
         if TEST_REPO:
-            cls.repo = Repository(TEMP_REPO_NAME, remote=TEST_REPO, autosync=False)
-            cls.wc = WorkingCopy(tempfile.mkdtemp(), repo=cls.repo)
+            cls.wc = WorkingCopy(tempfile.mkdtemp(), repo=TEMP_REPO)
 
     @classmethod
     def tearDownClass(cls):
         if TEST_REPO:
             cls.wc.delete()
-            cls.repo.delete()
 
     def test_init_existing_ws_instance(self):
         """Verify an item can be created from an existing WorkingCopy instance."""
@@ -361,143 +364,102 @@ class TestItem(unittest.TestCase):
 class TestTag(unittest.TestCase):
     """Tests on the Tag class."""  # pylint: disable=C0103
 
-    @classmethod
-    def setUpClass(cls):
-        if TEST_REPO:
-            cls.repo = Repository(TEMP_REPO_NAME, remote=TEST_REPO, autosync=False)
-
-    @classmethod
-    def tearDownClass(cls):
-        if TEST_REPO:
-            cls.repo.delete()
-
     def test_init(self):
         """Verify a tag can be created."""
-        tag = Tag('abc', repo=self.repo, rev='123')
+        tag = Tag('abc', repo=TEMP_REPO, rev='123')
         self.assertEqual('abc', tag)
         self.assertNotEqual(None, tag)
 
     def test_repo_tags(self):
         """Verify a tag can be added and removed from a repo."""
         try:
-            self.repo.add_tag('_temptag2', branch=Branch.MASTER)
-            tag = self.repo.add_tag('_temptag2', branch=Branch.MASTER)  # returns already created tag
+            TEMP_REPO.add_tag('_temptag2', branch=Branch.MASTER)
+            tag = TEMP_REPO.add_tag('_temptag2', branch=Branch.MASTER)  # returns already created tag
             log(tag)
-            self.assertIn(tag, self.repo.tags)
-            self.assertIn('_temptag2', self.repo.tags)
+            self.assertIn(tag, TEMP_REPO.tags)
+            self.assertIn('_temptag2', TEMP_REPO.tags)
         except:
             logging.error("exception will be raised after cleanup...")
             raise
         finally:
-            self.repo.remove_tag('_temptag2')
+            TEMP_REPO.remove_tag('_temptag2')
             tag.remove()  # already removed
 
     def test_repo_move_tag(self):
         """Verify a tag can be moved in a repository."""
-        try:
-            tag = self.repo.add_tag('_temptag', rev=2)
-            log(tag)
-            tag = tag.move(rev=1)
-            self.assertEqual(1, tag.rev)
-        except:
-            logging.error("exception will be raised after cleanup...")
-            raise
-        finally:
-            tag.remove()
+        tag = TEMP_REPO.add_tag('_temptag', rev=2)
+        log(tag)
+        tag = tag.move(rev=1)
+        self.assertEqual(1, tag.rev)
+        tag.remove()
 
     def test_no_changset(self):
         """Verify a revision, tag, or branch is required to add a tag."""
-        self.assertRaises(ValueError, self.repo.add_tag, '_invalid')
+        self.assertRaises(ValueError, TEMP_REPO.add_tag, '_invalid')
 
 
 @unittest.skipUnless(TEST_REPO, REASON)  # pylint: disable=R0904
 class TestStamp(unittest.TestCase):
     """Tests on the Stamp class."""  # pylint: disable=C0103
 
-    @classmethod
-    def setUpClass(cls):
-        if TEST_REPO:
-            cls.repo = Repository(TEMP_REPO_NAME, remote=TEST_REPO, autosync=False)
-
-    @classmethod
-    def tearDownClass(cls):
-        if TEST_REPO:
-            cls.repo.delete()
-
     def test_init(self):
         """Verify a stamp can be created."""
-        stamp = Stamp('abc', repo=self.repo, rev='123')
+        stamp = Stamp('abc', repo=TEMP_REPO, rev='123')
         self.assertEqual('abc', stamp)
         self.assertNotEqual(None, stamp)
 
     def test_repo_stamps(self):
         """Verify a stamp can be added and removed from a repo."""
-        try:
-            self.repo.add_stamp('_tempstamp', branch=Branch.MASTER)
-            stamp = self.repo.add_stamp('_tempstamp', branch=Branch.MASTER)  # returns already created stamp
-            log(stamp)
-            self.assertIn(stamp, self.repo.stamps)
-            self.assertIn('_tempstamp', self.repo.stamps)
-        except:
-            logging.error("exception will be raised after cleanup...")
-            raise
-        finally:
-            stamp.remove()
+        TEMP_REPO.add_stamp('_tempstamp', branch=Branch.MASTER)
+        stamp = TEMP_REPO.add_stamp('_tempstamp', branch=Branch.MASTER)  # returns already created stamp
+        log(stamp)
+        self.assertIn(stamp, TEMP_REPO.stamps)
+        self.assertIn('_tempstamp', TEMP_REPO.stamps)
+        stamp.remove()
 
     def test_no_changset(self):
         """Verify a revision, tag, or branch is required to add a stamp."""
-        self.assertRaises(ValueError, self.repo.add_stamp, '_invalid')
+        self.assertRaises(ValueError, TEMP_REPO.add_stamp, '_invalid')
 
 
 @unittest.skipUnless(TEST_REPO, REASON)  # pylint: disable=R0904
 class TestUser(unittest.TestCase):
     """Tests on the Tag class."""  # pylint: disable=C0103
 
-    @classmethod
-    def setUpClass(cls):
-        if TEST_REPO:
-            cls.repo = Repository(TEMP_REPO_NAME, remote=TEST_REPO, autosync=False)
-            cls.repo.add_user('_tempuser')
-
-    @classmethod
-    def tearDownClass(cls):
-        if TEST_REPO:
-            cls.repo.delete()
-
     def test_init(self):
         """Verify a user can be created."""
-        user = User('_tempuser', repo=self.repo)
+        user = User('_tempuser', repo=TEMP_REPO)
         log(user)
         self.assertEqual('_tempuser', user)
-        self.assertNotEqual(User('_tempuser2', repo=self.repo), user)
+        self.assertNotEqual(User('_tempuser2', repo=TEMP_REPO), user)
         self.assertNotEqual(None, user)
 
     def test_repo_users(self):
         """Verify the users list can be retrieved from a repository."""
-        log(self.repo.users)
-        self.assertIn('_tempuser', self.repo.users)
+        log(TEMP_REPO.users)
+        self.assertIn('_tempuser', TEMP_REPO.users)
 
     def test_repo_add_user_nobody(self):
         """Verify the 'nobody' user cannot be created."""
-        self.assertRaises(VeracityException, self.repo.add_user, User.NOBODY)
+        self.assertRaises(VeracityException, TEMP_REPO.add_user, User.NOBODY)
 
     def test_repo_add_user_existing(self):
         """Verify attempting to add an existing user is handled."""
-        user = self.repo.add_user('testuser')  # default user
+        user = TEMP_REPO.add_user('testuser')  # default user
         self.assertEqual('testuser', user)
 
     def test_repo_set_user(self):
         """Verify the current user can be set in a repository."""
-        self.repo.user = '_tempuser'
-        self.assertEqual('_tempuser', self.repo.user)
+        TEMP_REPO.user = '_tempuser'
+        self.assertEqual('_tempuser', TEMP_REPO.user)
 
     def test_repo_rename_user(self):
         """Verify a user can be renamed."""
-        u = User('testuser', repo=self.repo)
+        u = User('testuser', repo=TEMP_REPO)
         u.set()
         u = u.rename('testuser2')
         try:
-            self.assertEqual('testuser2', self.repo.user)
+            self.assertEqual('testuser2', TEMP_REPO.user)
         except:
             logging.error("exception will be raised after cleanup...")
             raise
@@ -506,7 +468,7 @@ class TestUser(unittest.TestCase):
 
     def test_repo_activate_user(self):
         """Verify a user can be deactivated and activated."""
-        u = User('testuser', repo=self.repo)
+        u = User('testuser', repo=TEMP_REPO)
         u.deactivate()
         u.deactivate()  # redundant
         u.activate()
@@ -517,16 +479,6 @@ class TestUser(unittest.TestCase):
 class TestBuildRequest(unittest.TestCase):
     """Tests on the BuildRequest class."""  # pylint: disable=C0103
 
-    @classmethod
-    def setUpClass(cls):
-        if TEST_REPO:
-            cls.repo = Repository(TEMP_REPO_NAME, remote=TEST_REPO, autosync=False)
-
-    @classmethod
-    def tearDownClass(cls):
-        if TEST_REPO:
-            cls.repo.delete()
-
     def test_init(self):
         """Verify a build request can be created."""
         request = BuildRequest('abc123', 'def456')
@@ -537,44 +489,44 @@ class TestBuildRequest(unittest.TestCase):
 
     def test_repo_request_build(self):
         """Verify a build can be requested in a repository."""
-        del self.repo.builds
-        cs = self.repo.get_heads(branch='master')[0]
-        request1 = self.repo.request_build('testbuild', 'C', 'W', 'Q', cs.rev, branch='master')
+        del TEMP_REPO.builds
+        cs = TEMP_REPO.get_heads(branch='master')[0]
+        request1 = TEMP_REPO.request_build('test build', 'C', 'W', 'Q', cs.rev, branch='master')
         log(request1)
-        self.assertEqual(BuildRequest(request1.bid, None, repo=self.repo), request1)
-        request2 = self.repo.get_build('W', 'Q')
+        self.assertEqual(BuildRequest(request1.bid, None, repo=TEMP_REPO), request1)
+        request2 = TEMP_REPO.get_build('W', 'Q')
         log(request2)
         self.assertEqual(request1, request2)
 
     def test_repo_request_builds(self):
         """Verify multiple builds can be requested in a repository."""
-        del self.repo.builds
-        cs = self.repo.get_heads(branch='master')[0]
-        self.repo.request_build('testbuild', 'C', 'W', 'Q', cs.rev)
-        self.repo.request_build('testbuild2', 'N', 'L', 'Q', cs.rev, branch='master')
-        self.assertEqual(2, len(list(self.repo.builds)))
+        del TEMP_REPO.builds
+        cs = TEMP_REPO.get_heads(branch='master')[0]
+        TEMP_REPO.request_build('test build', 'C', 'W', 'Q', cs.rev)
+        TEMP_REPO.request_build('test build 2', 'N', 'L', 'Q', cs.rev, branch='master')
+        self.assertEqual(2, len(list(TEMP_REPO.get_builds(limit=None))))
 
     def test_repo_request_build_invalid(self):
         """Verify an exception is raised when requesting an invalid build."""
-        self.assertRaises(VeracityScriptException, self.repo.request_build, '_', 'C', 'W', 'Q', 'fake')
+        self.assertRaises(VeracityScriptException, TEMP_REPO.request_build, '_', 'W', 'W', 'Q', 'fake')
 
     def test_repo_get_build_no_config(self):
         """Verify None is return when getting builds of an invalid environment."""
-        self.assertRaises(VeracityScriptException, self.repo.get_build, 'W', 'INVALID')
+        self.assertRaises(VeracityScriptException, TEMP_REPO.get_build, 'W', 'INVALID')
 
     def test_repo_get_build_no_env(self):
         """Verify None is return when getting builds of an invalid environment."""
-        self.assertIsNone(self.repo.get_build('INVALID', 'Q'))
+        self.assertIsNone(TEMP_REPO.get_build('INVALID', 'Q'))
 
     def test_repo_get_build_none(self):
         """Verify nothing is returned when no builds are queued."""
-        del self.repo.builds
-        self.assertEqual(None, self.repo.get_build('M', 'Q'))
+        del TEMP_REPO.builds
+        self.assertEqual(None, TEMP_REPO.get_build('M', 'Q'))
 
     def test_update(self):
         """Verify a build request can be updated."""
-        cs = self.repo.get_heads(branch='master')[0]
-        request = self.repo.request_build('testbuild3', 'M', 'M', 'Q', cs.rev)
+        cs = TEMP_REPO.get_heads(branch='master')[0]
+        request = TEMP_REPO.request_build('testbuild3', 'M', 'M', 'Q', cs.rev)
         log(request)
         self.assertEqual('Q', request.status)
         request.update('B')
@@ -583,8 +535,8 @@ class TestBuildRequest(unittest.TestCase):
 
     def test_delete(self):
         """Verify a build request can be deleted."""
-        cs = self.repo.get_heads(branch='master')[0]
-        request = self.repo.request_build('testbuild4', 'C', 'L', 'Q', cs.rev)
+        cs = TEMP_REPO.get_heads(branch='master')[0]
+        request = TEMP_REPO.request_build('testbuild4', 'C', 'L', 'Q', cs.rev)
         log(request)
         request.delete()
         self.assertEqual(None, request.status)
